'This BMX file was edited with BLIde ( http://www.blide.org )
Rem
	bbdoc: The basic class of all gui items
End Rem
Type TGuiObject Abstract
	Global EVENT_BUTTONPRESSED:Int
	Global EVENT_BUTTONRELEASED:Int
	Global EVENT_BUTTONCLICKED:Int
	Global EVENT_MOUSEOVER:Int
	Global EVENT_MOUSELEFT:Int
	
	Global MouseX:Int
	Global MouseY:Int

	'gets called only once by TProgramManager on programStart to add Event-ID's
	Function _onProgramStart()
		EVENT_BUTTONPRESSED = AllocUserEventId("A button of the gui got pressed (mouse over and mouse down")
		EVENT_BUTTONRELEASED = AllocUserEventId("A button of the gui got released (mouse over and mouse down, then mouse up or mouse not over")
		EVENT_BUTTONCLICKED = AllocUserEventId("A button of the gui got clicked (mouse over and mouse down, then mouse over and mouse up")
		EVENT_MOUSEOVER = AllocUserEventId("The cursor was moved above this object")
		EVENT_MOUSELEFT = AllocUserEventId("The cursor left this object")
	End Function

	'parent gui object
	Field _parent:TGuiObject
	Field _parentLink:TLink
	'List of all children:
	Field _children:TList
	
	'absolute handle in pixels, datatype double for smooth animations. This value is substracted from the pixelPosition. Handle = 0 means top left of this box.
	Field _absoluteHandleX:Double
	Field _absoluteHandleY:Double
	
	'relative to its own size (width, height), value * width OR value * height, so value should be between 0 and 1, 1 meaning the handle is on the right/bottom edge of this object.
	Field _relativeHandleX:Double
	Field _relativeHandleY:Double
	
	'absolute position in pixels (always relative to parent!)
	Field _absolutePositionX:Double
	Field _absolutePositionY:Double
	
	'relative position to parent size (valueOf(_relativePositionX) * Parent.width + _absolutePositionX = pixelCoordinateX) (always relative to parent!)
	Field _relativePositionX:Double
	Field _relativePositionY:Double
	
	Field _mouseOverNeedsUpdate:Byte = True
	Field _positionNeedsUpdate:Byte = True
	
	'real left&top pixel position of this object (calculated by calculatePosition() using all other size, handle and position coordinates)
	Field X:Double
	Field Y:Double
	
	'own size in pixels
	Field width:Double
	Field height:Double
	
	'name of this object
	Field name:String
	
	'some state variables
	Field visible:Byte = True
	Field active:Byte = True
	
	'some event variables
	Field state_mouseOver:Byte
	
	Method setName(name:String)
		Self.name = name
	End Method

	Rem
		bbdoc: set the handle of this object. Relative = to its own size
	end rem
	Method SetHandle(XinPixel:Double, YinPixel:Double, Xrelative:Double, Yrelative:Double)
		_absoluteHandleX = XinPixel
		_absoluteHandleY = YinPixel
		_relativeHandleX = Xrelative
		_relativeHandleY = Yrelative
		_positionNeedsUpdate = True
	End Method
	
	Method midhandle()
		SetHandle(0, 0, 0.5, 0.5)
	End Method
	
	Method setSize(width:Double, height:Double)
		Self.width = width
		Self.height = height
		_positionNeedsUpdate = True
	End Method
	
	Rem
		bbdoc: set the position of this object. Relative = to its parent size. If a parent is available, the position is always relative to its parents position (means, if bot have a absolute 
	end rem
	Method SetPosition(XinPixel:Double, YinPixel:Double, Xrelative:Double, Yrelative:Double)
		_absolutePositionX = XinPixel
		_absolutePositionY = YinPixel
		_relativePositionX = Xrelative
		_relativePositionY = Yrelative
		_positionNeedsUpdate = True
	End Method
	
	Rem
		bbdoc: calculate position of this object if anything got changed.
	end rem
	Method calculatePosition()
		X = _absolutePositionX - _absoluteHandleX - _relativeHandleX * width
		Y = _absolutePositionY - _absoluteHandleY - _relativeHandleY * height
		If _parent
			X:+_parent.X + _relativePositionX * _parent.width
			Y:+_parent.Y + _relativePositionY * _parent.height
		Else 'use the value of the screen coordinates
			X:+_relativePositionX * TProgramSettings.GFX_WIDTH
			Y:+_relativePositionY * TProgramSettings.GFX_HEIGHT
		EndIf
		If _children
			For Local o:TGuiObject = EachIn _children
				o._positionNeedsUpdate = True
			Next
		EndIf
		_positionNeedsUpdate = False
		_mouseOverNeedsUpdate = True
	End Method
	
	Rem
		bbdoc: add a child object to this object. A child gets drawn and processed when the parent is drawn and processed.
	end rem
	Method add(child:TGuiObject)
		Assert Not (child = Null), "TGuiObject.add(TGuiObject child) child = null"
		If _children = Null _children = New TList
		'if there is already a parent assigned to this child:
		If child._parent child.removeParent()
		'set this object as the parent of the child
		child._parentLink = _children.AddLast(child)
		child._parent = Self
		child._positionNeedsUpdate = True
	End Method
	
	Rem
		bbdoc: get rid of your parent!
	end rem
	Method removeParent()
		If Self._parent = Null Return
		Self._parent = Null
		If Self._parentLink = Null Return
		Self._parentLink.Remove()
		Self._parentLink = Null
	End Method
	
	Rem
		bbdoc: a visible object is visible (somehow this is logically, isn't it? ;) )
	End Rem
	Method setVisible(value:Byte = True)
		visible = value
	End Method
	
	Rem
		bbdoc: an active object processes events and is shown enabled.
	End Rem
	Method setActive(value:Byte = True)
		active = value
	End Method
	
	
	Rem
		bbdoc: This one gets called before actual drawing takes place. It returns "true" if the drawing command shouldn't be executed.
	End Rem
	Method preDraw:Byte()
		If Self.visible = False Return True
		If _positionNeedsUpdate calculatePosition()
		If _mouseOverNeedsUpdate _updateMouseOver()
		Return False
	End Method
	
	Rem
		bbdoc: Draws this object and all of its children
		call "super.draw() at the end of your implementation to make sure, all children will be drawn over this object. make sure you check self.visible = true before drawing!
		write "if preDraw() return" at the beginning of your own implementation to break if this item is visible and to calculate some things before actual drawing.
	End Rem
	Method draw()
		If preDraw() Return
		If _children
			For Local o:TGuiObject = EachIn _children
				o.draw()
			Next
		EndIf
	End Method
	
	Rem
		bbdoc: Processes this object and all of its children. Make sure your call self.active = true before processing!
		call "super.process() at the end of your implementation to make sure, all children will be processed"
	End Rem
	Method process()
		If Self.active = False Return
		If _children
			For local o:TGuiObject = EachIn _children
				o.process()
			Next
		EndIf
	End Method
	
	Rem
		bbdoc: Gets called if an event occurs. return "null" if this object processed the event. Otherwise return super.onEvent() to make sure your children get it as well.
		if your object is not active and is not affected by the event, return the event so that other objects that might need it can use it.
	end rem
	Method onEvent:Object(id:Int, event:TEvent, context:Object)
		If Self.active = False Return event
		Select event.id
			Case EVENT_MOUSEMOVE
				TGuiObject.MouseX = event.X
				TGuiObject.MouseY = event.Y
				_mouseOverNeedsUpdate = True
				Return event
		End Select
	
		If _children
			For local o:TGuiObject = EachIn _children
				If o.onEvent(id, event, context) = Null Return Null
			Next
		EndIf
		Return event 'if we dont need this event, we pass it back so others can use it.
	End Method
	
	
	Rem
		bbdoc: updates mouseOver status of this gui element.
	end rem
	Method _updateMouseOver()
		_mouseOverNeedsUpdate = False
		Local before:Byte = state_mouseOver
		If TGuiObject.MouseX > X And TGuiObject.MouseX < X + Self.width And TGuiObject.MouseY > Y And TGuiObject.MouseY < Y + Self.height
			state_mouseOver = True
		ElseIf state_mouseOver = True
			state_mouseOver = False
		Else 'if the mouse is not over this object and it also wasnt before, it also wasnt over any children of this object.
			Return
		EndIf
		If _children
			For local o:TGuiObject = EachIn _children
				o._mouseOverNeedsUpdate = True
			Next
		EndIf
		If before = False And state_mouseOver = True
			onMouseOver()
		End If
		
		If before = True And state_mouseOver = False
			onMouseLeft()
		End If
	End Method
	
	rem
		bbdoc: The mousecursor enters this objects surface
	end rem
	Method onMouseOver()
		Local e:TEvent = CreateEvent:TEvent(EVENT_MOUSEOVER, Self)
		e.Emit()
	End Method
	
	rem
		bbdoc: The mousecursor left this object's surface
	end rem
	Method onMouseLeft()
		Local e:TEvent = CreateEvent:TEvent(EVENT_MOUSELEFT, Self)
		e.Emit()
	End Method
End Type
