﻿'This BMX file was edited with BLIde ( http://www.blide.org )
Rem
	bbdoc: a class to manage and draw particles.
	
	Konzept:
	TParticleEmitter verarbeitet die Partikel. Da Partikel in unterschiedlichsten Formen und Geschwindigkeiten gespawnt werden sollen, wird das ausgelagert:
	
	Die Partikel selbst bekommen ein paar Grundlegende Eigenschaften. Wenn man jetzt eine Explosion erstellen möchte, dann leitet man sich einen "TParticleEmitter" ab und implementiert die Spawn-Methode selbst.
	
	
End Rem
Type TParticleEmitter Abstract

	'recycling pool of not used particles, using arrays for lower overhead (list-links would create overhead that needs to be processed by the GC)
	Global particlePool:TParticle[100]
	Global particlePoolPointer:Int = -1
	Global list:TList = New TList
	Global drawOffsetX:Double, drawOffsetY:Double
	
	Function init()
		particlePool = New TParticle[100]
		particlePoolPointer = -1
		For Local pe:TParticleEmitter = EachIn list
			 pe.remove()
		Next
		list.Clear()
	End Function
	
	Function LoadColorGradient:Int[,] (url:Object)
		Local image:TPixmap = LoadPixmapPNG:TPixmap(url)
		Local colorArray:Int[,] = New Int[image.width, 4]
		For Local x:Int = 0 Until image.width
			Local color:Int = image.ReadPixel(x, 0)
			colorArray[x, 0] = (color & $ff000000) Shr 24
			colorArray[x, 1] = (color & $ff0000) Shr 16
			colorArray[x, 2] = (color & $ff00) Shr 8
			colorArray[x, 3] = color & $ff
		Next
		Return colorArray
	End Function
	
	Function ApplyColorGradient(arr:Int[,], progress:Double)
		Local frame:Double = progress * (arr.length * 0.25 - 1)
		Local invertFrame:Double

		'for linear interpolation:
		Local first:Int = Floor(frame)
		Local second:Int = Ceil(frame)
		
		frame = frame Mod 1
		invertFrame = 1.0 - frame
		
		Local alpha:Double = (invertFrame * arr[first, 0] + frame * arr[second, 0]) / $100
		Local red:Int = (invertFrame * arr[first, 1] + frame * arr[second, 1])
		Local green:Int = (invertFrame * arr[first, 2] + frame * arr[second, 2])
		Local blue:Int = (invertFrame * arr[first, 3] + frame * arr[second, 3])
		SetAlpha alpha
		SetColor red, green, blue
	End Function
	
	Rem
		bbdoc: Returns a particle type (either using and old one form the recycling heap or creating a new one)
	end rem
	Function getNewParticle:TParticle()
		Local p:TParticle
		If particlePoolPointer >= 0
			p = particlePool[particlePoolPointer]
			particlePool[particlePoolPointer] = Null
			particlePoolPointer:-1
		Else
			p = New TParticle
		EndIf
		p.x = 0
		p.y = 0
		p.r = 0
		p.sizex = 1
		p.sizey = 1
		p.speedx = 0
		p.speedy = 0
		p.speedr = 0
		p.accelerationx = 0
		p.accelerationy = 0
		p.accelerationr = 0
		p.dampeningx = 1
		p.dampeningy = 1
		p.dampeningr = 1
		p.totallifetime = 1000
		p.lifetime = p.totallifetime
		Return p
	End Function
	
	Rem
		bbdoc: add a particle that is not used anymore to the recycling-heap.
	end rem
	Function recycleParticle(p:TParticle)
		particlePoolPointer:+1
		If particlePoolPointer >= particlePool.length
			particlePool = particlePool[..(particlePool.length + 100)]'increase heapsize
		EndIf
		particlePool[particlePoolPointer] = p
	End Function
	
	Function processAll(dt:Double)
		For Local pe:TParticleEmitter = EachIn list
			If Not pe.waiting
				 pe.dt:+dt
				 pe.processSpawn(dt)
			EndIf
		Next
	End Function
	
	Function drawAll(x:Double = 0, y:Double = 0)
		drawOffsetX = x
		drawOffsetY = y
		For Local pe:TParticleEmitter = EachIn list
			 If Not pe.waiting pe.drawParticles()
		Next
		SetRotation 0
		SetColor 255, 255, 255
		SetBlend ALPHABLEND
	End Function
	
	Field x:Double, y:Double
	Field _link:TLink
	Field dt:Double
	Field activeParticles:TParticle[]
	Field activeParticlesPointer:Int = -1
	Field particleCounter:Int
	
	Field particleimage:TImage
	Field colorgradient:Int[,]
	Field blendmode:Byte = ALPHABLEND
	
	Field waiting:Byte
	
	Method New()
		Self._link = TParticleEmitter.list.AddLast(Self)
	End Method
	
	Method go()
		waiting = False
	End Method
	
	Method wait()
		waiting = True
	End Method
	
	Method processSpawn(dt:Double) Abstract
	
	Rem
		bbdoc: Process, Sort and Render the particle array:
	end rem
	Method drawParticles()
		Local p:TParticle
		Local particleCounter:Int = -1
		activeParticlesPointer:Int = -1
		SetBlend blendmode
		SetAlpha 1
		SetColor 255, 255, 255
		
		For Local i:Int = 0 Until activeParticles.length
			p = activeParticles[i]
			
			'Check if lifetime of particle is over
			If p
				p.lifetime:-dt
				If p.lifetime < 0
					activeParticles[i] = Null
					recycleParticle(p)
					p = Null
				EndIf
			EndIf
			
			'Compress particles:
			If p = Null And particleCounter < 0 particleCounter = i
			If p And particleCounter > 0
				activeParticles[particleCounter] = p
				activeParticles[i] = Null
				particleCounter:+1
			EndIf
			
			If p
				activeParticlesPointer:+1
				'process particle
				p.x:+dt * (p.speedx + 0.5 * p.accelerationx * dt)
				p.y:+dt * (p.speedy + 0.5 * p.accelerationy * dt)
				p.y:+dt * (p.speedr + 0.5 * p.accelerationr * dt)
				p.speedx:+p.accelerationx * dt
				p.speedy:+p.accelerationy * dt
				p.speedr:+p.accelerationr * dt
				p.speedx:*p.dampeningx
				p.speedy:*p.dampeningy
				p.speedr:*p.dampeningr
				ApplyColorGradient(colorgradient, 1.0 - (p.lifetime / p.totallifetime))
				SetRotation p.r
				SetScale p.sizex, p.sizey
				DrawImage particleimage, p.x + drawOffsetX, p.y + drawOffsetY
			EndIf
		Next
		SetColor 255, 255, 255
		SetBlend ALPHABLEND
		SetRotation 0
		SetScale 1, 1
		dt = 0
	End Method
	
	'always compressArray after you removed one or more particles from the array (better you first remove all you need and then once call this)
	'DEPRECIATED
	Method compressArray()
		Local p:TParticle
		activeParticlesPointer = -1
		
		For Local i:Int = 0 Until activeParticles.length
			p = activeParticles[i]
			'sort:
			If p = Null And activeParticlesPointer < 0 activeParticlesPointer = i
			If p And activeParticlesPointer > 0
				activeParticles[activeParticlesPointer] = p
				activeParticles[i] = Null
				activeParticlesPointer:+1
			EndIf	
		Next
	End Method
	
	Method addParticle(p:TParticle)
		activeParticlesPointer:+1
		If activeParticlesPointer >= activeParticles.Length
			'increase size
			activeParticles = activeParticles[..(activeParticles.Length + 100)]
		End If
		activeParticles[activeParticlesPointer] = p
	End Method
	
	Method remove()
		activeParticles = Null
		RemoveLink(_link)
	End Method
End Type
