Function checkConsoleActions%()
  If AppArgs.length = 1 Then Return 0
  Select AppArgs[1]
   Case "-createMap"
    If AppArgs.length <> 6 Then
      showHelp "createMap"
      Return 1
    EndIf
    wayPoints = New TList
    Local filename$ = AppArgs[2]
    bgFileName = AppArgs[3]
    Local wayPointStrs$[] = AppArgs[4].split(";")
    Local buildLockStrs$[] = AppArgs[5].split(";")
    For Local str$ = EachIn wayPointStrs
      Local parts$[] = str.split(",")
      If parts.length <> 2 Then
        showHelp "createMap"
        Return 1
      EndIf
      TWayPoint.Create(Int(parts[0]), Int(parts[1]))
    Next

    For Local str$ = EachIn buildLockStrs
      Local parts$[] = str.split(",")
      If parts.length <> 2 Then
        showHelp "createMap"
        Return 1
      EndIf
      Local x% = Int(parts[0]), y% = Int(parts[1])
      If x < 0 Or x > TILESET_WIDTH-1 Or y < 0 Or y > TILESET_HEIGHT-1 Then
        showHelp "createMap"
        Return 1
      EndIf
      buildLock[x][y] = True
    Next

    Local stream:TStream = WriteFile(filename)
    saveMap stream
    CloseFile stream
    Return 1
   Case "-mapEditor"
    If AppArgs.length > 3 Then
      showHelp "mapEditor"
      Return 1
    EndIf
    init
    initElements
    If AppArgs.length = 3 Then
      If FileType(AppArgs[2]) <> 1 Then
        showHelp "mapEditor"
        Return 1
      EndIf
      Local stream:TStream = ReadFile(AppArgs[2])
      loadMap stream
      CloseFile stream
    EndIf
    showEditor
    Return 1
   Case "-waveEditor"
    If AppArgs.length > 3 Then
      showHelp "waveEditor"
      Return 1
    EndIf
    init
    initElements
    If AppArgs.length = 3 Then
      If FileType(AppArgs[2]) <> 1 Then
        showHelp "waveEditor"
        Return 1
      EndIf
      Local stream:TStream = ReadFile(AppArgs[2])
      currentWave = loadWave(stream)
      CloseFile stream
    EndIf
    editWave
    Return 1
   Case "-levelEditor"
    If AppArgs.length > 3 Then
      showHelp "levelEditor"
      Return 1
    EndIf
    initElements
    If AppArgs.length = 3 Then
      If FileType(AppArgs[2]) <> 1 Then
        showHelp "levelEditor"
        Return 1
      EndIf
      Local stream:TStream = ReadFile(AppArgs[2])
      currentLevel = loadLevel(stream)
      CloseFile stream
    EndIf
    levelEditor
    Return 1
   Default
    Print "param "+AppArgs[1]+" is unsopported"
    Print ""
    Print "supported params: "
    Print AppFile+" -createMap filename background waypoints buildlocks"
    Print AppFile+" -mapEditor [filename]"
    Print AppFile+" -waveEditor [filename]"
    Return 1
  End Select
End Function

Function showHelp(funcname$)
  Select funcname
   Case "createMap"
    Print "Syntax-error of param -createMap"
    Print ""
    Print AppFile+" -createMap filename background waypoints buildlocks"
    Print ""
    Print "  filename:   The name of the file to save the map"
    Print "              ATTENTION: any existing file of name filename will be overwritten!"
    Print "  background: The filename of the background image ("+TILESET_WIDTH*TILE_SIZE+"x"+..
              TILESET_WIDTH*TILE_SIZE+")"
    Print "  waypoints:  Semicolon-separated list of waypoints (comma separated) in quotes"
    Print "  buildlocks:  Semicolon-separated list of build-locked tiles (comma separated) in quotes"
    Local samp$ = AppFile+" mymap.dat gfx/bg.png "+Chr(34)+"10,10;100,10;10,100;100,100"+Chr(34)+" "+..
            Chr(34)+"0,0;1,0;0,1;1,1"+Chr(34)
    Print "example: >"+samp
    Print "Saves a map as mymap.dat, which background-image is located at gfx/bg.png"
    Print "The enemies will go to the points (10|10), (100|10), (10|100), (100|100) in order"
    Print "It wont be possible to build at (0|0), (1|0), (0|1), (1|1)"
   Case "mapEditor"
    Print "The given arguments are too many or invalid."
    Print ""
    Print AppFile+" -mapEditor [filename]"
    Print "Opens the graphical map editor. If given, the map saved as filename will be loaded."
   Case "waveEditor"
    Print "The given arguments are too many or invalid."
    Print ""
    Print AppFile+" -waveEditor [filename]"
    Print "Opens the graphical wave editor. If given, the wave saved as filename will be loaded."
   Case "levelEditor"
    Print "The given arguments are too many or invalid."
    Print ""
    Print AppFile+" -levelEditor [filename]"
    Print "Opens the level editor. If given, the level saved as filename will be loaded."
  End Select
End Function