Type TEnemyKind
  Global list:TList = New TList
  Field speed#, health%, size%, image:TImage
  Field id%
  Method New()
    Self.id = Self.list.Count()
    Self.list.addLast Self
  End Method
  Field draw(instance:TEnemy)
  Method getMoney%()
    Return Ceil(Float(Self.health)/10)
  End Method
  Method getDestruction%()
    Return Ceil(Float(Self.health)/10)
  End Method
End Type
Global wayPoints:TList = New TList
Type TEnemy
  Global list:TList = New TList
  Field x#, y#, health%, currentAim:TWayPoint, currentAimIndex%, moved#
  Field dead% = False
  Field kind:TEnemyKind
  Method New()
    Self.list.addLast Self
  End Method
  Method init()
    Self.health = Self.kind.health
    Self.currentAim = TWayPoint(wayPoints.First())
    Self.x = Self.currentAim.x
    Self.y = Self.currentAim.y
    Self.currentAim = TWayPoint(wayPoints.valueAtIndex(1))
    Self.currentAimIndex = 1
  End Method
  Function Create:TEnemy(kind:TEnemyKind)
    Local result:TEnemy = New TEnemy
    result.kind = kind
    result.init
    Return result
  End Function

  Method draw()
    Self.kind.draw Self
  End Method

  Method setXY()
    Local pos%[] = Self.currentAim.getPosition(moved)
    Self.x = pos[0]
    Self.y = pos[1]
  End Method
  Method getDir#()
    Return Self.currentAim.moveDir
  End Method
  Method update()
    Self.moved :+ Self.kind.speed
    Local pos%[] = Self.currentAim.getPosition(moved)
    If pos.length < 2 Then
      Self.currentAimIndex :+ 1
      moved = 0
      If Self.currentAimIndex >= wayPoints.Count() Then
        Self.enterAim
        Return
      EndIf
      Self.currentAim = TWayPoint(wayPoints.valueAtIndex(Self.currentAimIndex))
    Else
      Self.x = pos[0]
      Self.y = pos[1]
    EndIf
  End Method
  Method enterAim()
    lives :- Self.kind.getDestruction()
    destruction :+ Self.kind.getDestruction()
    Self.destroy
  End Method
  Method destroy()
    Self.dead = True
    Self.list.remove Self
  End Method

  Method checkHit(power%)
    Self.health :- power
    If Self.health <= 0 Then Self.die
  End Method
  Method die()
    money :+ Self.kind.getMoney()
    Self.destroy
  End Method


  Function drawAll()
    For Local enemy:TEnemy = EachIn Self.list
      enemy.draw
    Next
  End Function

  Function updateAll()
    For Local enemy:TEnemy = EachIn Self.list
      enemy.update
    Next
  End Function

End Type

Type TWayPoint
  Field x%, y%, moveDir#, previous:TWayPoint, distance%
  Function Create:TWayPoint(x%, y%, previous:TWayPoint = Null)
    Local result:TWayPoint = New TWayPoint
    result.x = x
    result.y = y
    If Not previous Then
      If Not wayPoints.isEmpty() Then result.previous = TWayPoint(wayPoints.Last())
    Else
      result.previous = previous
    EndIf
    result.calcFields
    wayPoints.addLast result
    Return result
  End Function
  Method calcFields()
    If Not Self.previous Then Return
    Self.moveDir = ATan2(Self.y-Self.previous.y, Self.x-Self.previous.x)+90
    Self.distance = Sqr((Self.x-Self.previous.x)^2+(Self.y-Self.previous.y)^2)
  End Method
  Method getPosition%[](moved#)
    If moved > Self.distance Or Not Self.previous Then
      Local result%[0]
      Return result
    EndIf
    Local rel# = (moved/Self.distance)
    Local result%[2]
    If Self.x <> Self.previous.x Then
      result[0] = Self.previous.x+(Self.x-Self.previous.x)*rel
    Else
      result[0] = Self.x
    EndIf
    If Self.y <> Self.previous.y Then
      result[1] = Self.previous.y+(Self.y-Self.previous.y)*rel
    Else
      result[1] = Self.y
    EndIf
    Return result
  End Method
End Type
Function simpleDrawEnemy(enemy:TEnemy)
  SetRotation enemy.getDir()
  SetColor 255, 255, 255
  DrawImage enemy.kind.image, enemy.x, enemy.y
  SetRotation 0
End Function
