Const sx% = 800, sy% = 600

Global tooltip$ = "", lastMouseMove%

Const DIR_UP%    = 0
Const DIR_RIGHT% = 1
Const DIR_DOWN%  = 2
Const DIR_LEFT%  = 3

Const TILESET_WIDTH% = 35
Const TILESET_HEIGHT% = 28

Const MENU1_WIDTH% = 100
Const MENU2_HEIGHT% = 40

Const TILE_SIZE% = 20
Const CURSOR_WIDTH% = 20, CURSOR_HEIGHT% = 20

Global mx%, my%
Global cursorImage:TImage

Const PLAYER_DIR$ = "players/"
Const LEVEL_DIR$ = "levels/"
Const STORY_DIR$ = LEVEL_DIR+"story/"
Const IMG_DIR$ = "gfx/"

Global currentWave:TWave, currentLevel:TLevel
Global currentPlayer:TPlayer

Global destruction% = 0

Const WIRE_ALPHA# = .7
Global buildLock:Byte[][]
buildLock = buildLock[..TILESET_WIDTH]
For Local i%=0 To TILESET_WIDTH-1
  buildLock[i] = buildLock[i][..TILESET_HEIGHT]
Next
Global bg:TImage, bgFileName$
Global bgImages:TMap = CreateMap()

Global generalBG:TImage, levelEnabled:TImage, levelDisabled:TImage, logoImg:TImage
Global moneyImg:TImage, livesImg:TImage, noElectricityImg:TImage

Const START_MONEY% = 4000, START_LIVES% = 10
Global money% = START_MONEY, lives% = START_LIVES

Function updateEverything()
  TWire.updateAll
  TTower.updateAll
  TEnemy.updateAll
  TBullet.updateAll
  TDetonation.updateAll
End Function

Function drawEverything()
  setGameDraw
  Cls
  SetColor 255, 255, 255
  SetBlend MASKBLEND
  DrawImage bg, 0, 0
  SetBlend ALPHABLEND
  TWire.drawAll
  If playerUpgradeTower Then
    SetColor 255, 255, 255
    SetAlpha .4
    Local range% = playerUpgradeTower.getRange()
    Local add% = TILE_SIZE/2-range
    DrawOval playerUpgradeTower.x*TILE_SIZE+add, playerUpgradeTower.y*TILE_SIZE+add, range*2, range*2
    SetAlpha 1
  EndIf
  TTower.drawAll
  TEnemy.drawAll
  TBullet.drawAll
  TDetonation.drawAll
  If playerChoosenTower Then
    SetColor 255, 255, 255
    SetAlpha .4
    DrawOval mx-playerChoosenTower.range-MENU1_WIDTH, my-playerChoosenTower.range,..
             playerChoosenTower.range*2, playerChoosenTower.range*2
    SetAlpha 1
    DrawImage playerChoosenTower.image, mx-MENU1_WIDTH, my
  ElseIf playerChoosenWire Then
    SetColor 255, 255, 255
    DrawImage playerChoosenWire.image, mx-MENU1_WIDTH, my
  EndIf

  If Not currentWave And TEnemy.list.isEmpty() Then
    SetColor 0, 255, 0
    DrawText "Press [Space] to start wave", 0, 0
  EndIf
  Flip
End Function

Global lastFrame% = 0, FPS% = 0, frameCnt% = 0
Function handleFPS()
  If lastFrame+1000 <= MilliSecs() Then
    lastFrame = MilliSecs()
    FPS = frameCnt
    frameCnt = 0
  EndIf
  frameCnt :+ 1
End Function


Global timer:TTimer
Global terminatedWave:TWave = Null
Function playGame()
  unsetViewport
  Cls
  Flip
  Cls
  timer = CreateTimer(30)
  Local drawM2% = False, gotMoney% = True
  Repeat
    WaitEvent
    Select EventID()
     Case EVENT_TIMERTICK
      updateEverything
      If drawM2 Then
        drawMenu2
        drawM2 = False
      EndIf
      drawMenu1
      drawEverything
      handleFPS
      If Not TEnemy.list.isEmpty() Then
        gotMoney = False
      ElseIf terminatedWave Then
        terminatedWave.stop
        terminatedWave = Null
        gotMoney = True
      EndIf
      If currentWave Then
        If currentWave.releaseEnemy() Then
          terminatedWave = currentWave
          currentWave = Null
        EndIf
      ElseIf Not currentLevel.hasNextWave() And TEnemy.list.isEmpty() Then
        showWon
        If currentLevel.id <> -1 And currentPlayer Then currentPlayer.winLevel currentLevel.id
        Exit
      EndIf
     Case EVENT_APPTERMINATE
      End
     Case EVENT_KEYDOWN
      If EventData() = KEY_ESCAPE Then Exit
      If Not currentWave And EventData() = KEY_SPACE And TEnemy.list.isEmpty() Then
        If Not currentLevel.startWave() Then
          showWon
          If currentLevel.id And currentPlayer Then currentPlayer.winLevel currentLevel.id
          Exit
        EndIf
        gotMoney = True
      EndIf
     Case EVENT_MOUSEMOVE
      mx = EventX()
      my = EventY()
    End Select
    Select checkPlayerAction()
     Case 1
      drawM2 = True
     Case -1
      Exit
    End Select
    If lives <= 0 Then
      showLoose
      Exit
    EndIf
  Forever
  StopTimer timer
  timer = Null
End Function

Function showWon()
  unsetViewport
  Cls
  DrawText "You won!", 0, 0
  Flip
  Repeat
    WaitEvent
    Select EventID()
     Case EVENT_MOUSEMOVE
      mx = EventX()
      my = EventY()
      Cls
      DrawText "You won!", 0, 0
      Flip
     Case EVENT_KEYDOWN
      Return
     Case EVENT_APPTERMINATE
      End
    End Select
  Forever
End Function

Function showLoose()
  unsetViewport
  Cls
  DrawText "You lost!", 0, 0
  Flip
  Repeat
    WaitEvent
    Select EventID()
     Case EVENT_MOUSEMOVE
      mx = EventX()
      my = EventY()
      Cls
      DrawText "You lost!", 0, 0
      Flip
     Case EVENT_KEYDOWN
      Return
     Case EVENT_APPTERMINATE
      End
    End Select
  Forever
End Function

Function loadGeneralGFX()
  generalBG = LoadImage(IMG_DIR+"bg.png")
  SetImageHandle generalBG, 0, 0
  levelEnabled = LoadImage(IMG_DIR+"levelEnabled.png")
  levelDisabled = LoadImage(IMG_DIR+"levelDisabled.png")
  logoImg = LoadImage(IMG_DIR+"logo.png")
  SetImageHandle logoImg, 0, 0
  noElectricityImg = LoadImage(IMG_DIR+"noElectricity.png")
  SetImageHandle noElectricityImg , 0, 0
  moneyImg = LoadImage(IMG_DIR+"money.png")
  SetImageHandle moneyImg, 0, 0
  livesImg = LoadImage(IMG_DIR+"heart.png")
  SetImageHandle livesImg, 0, 0
End Function

Function startGame()
  showIntro
  reset
  loadGeneralGFX
  TLevel.loadAll
  TPlayer.loadAll
  If showMenu() = -1 Then Return
  Repeat
    choosePlayer
    If Not currentPlayer Then
      Exit
    EndIf
    Repeat
      chooseLevel
      If Not currentLevel Then
        Exit
      EndIf
      currentLevel.activateMap
      playGame
      reset
    Forever
    TPlayer.saveAll
  Forever
End Function

Function reset()
  TTower.list.Clear
  TEnemy.list.Clear
  TBullet.list.Clear
  TWire.list.Clear
  TDetonation.list.Clear
  wayPoints.Clear
  money = START_MONEY
  lives = START_LIVES
  currentWave = Null
  playerChoosenTower = Null
  playerChoosenWire  = Null
  playerUpgradeTower = Null
  terminatedWave = Null
  destruction = 0
  If currentLevel Then
    currentLevel.currentWaveIndex = -1
  EndIf
  For Local x%=0 Until (sx-MENU1_WIDTH)/TILE_SIZE
    For Local y%=0 Until (sy-MENU2_HEIGHT)/TILE_SIZE
      TTower.index[x][y] = Null
      TWire.index[x][y] = Null
      buildLock[x][y] = 0
    Next
  Next
  tooltip = ""
End Function

Function setTooltip(t$)
  If lastMouseMove+300 < MilliSecs() Then tooltip = t
End Function
Function tolltipMouseMove()
  lastMouseMOve = MilliSecs()
  tooltip = ""
End Function
Function playUserLevel()
  Local file$ = RequestFile("Select level-file")
  If Not file Then Return
  Local stream:TStream = ReadFile(file)
  currentLevel = loadLevel(stream)
  CloseFile stream
  currentLevel.activateMap
  playGame
  reset
End Function