Type TStep
  Field pauseBefore%, enemies:TList = New TList, enemyTime#, randomize%, tmp:TList
  Field frames%, releasedEnemies%
  Function Create:TStep(pauseBefore%, enemyTime#, randomize%)
    Local result:TStep = New TStep
    result.pauseBefore = pauseBefore
    result.enemyTime = enemyTime
    result.randomize = randomize
    Assert enemyTime > 0.0, "Invalid enemy-time given"
    Return result
  End Function
  Method addEnemy(enemy:TEnemyKind)
    Self.enemies.addLast enemy
  End Method
  Method start()
    Self.tmp = Self.enemies.Copy()
    If Self.randomize Then
      Self.tmp.Sort True, Self.__randomSort
    EndIf
    Self.frames = 0
    Self.releasedEnemies = 0
  End Method
  Method releaseEnemy%()
    If Self.tmp.isEmpty() Then
      Self.tmp = Null
      Return 1
    EndIf
    Self.frames :+ 1
    If Self.frames <= Self.pauseBefore Then Return 0
    Local no% = Float(Self.frames-Self.pauseBefore)/Self.enemyTime-Self.releasedEnemies
    For Local i%=1 To no
      TEnemy.Create TEnemyKind(Self.tmp.removeFirst())
      If Self.tmp.isEmpty() Then
        Self.tmp = Null
        Return 1
      EndIf
    Next
    Self.releasedEnemies :+ no
    Return 0
  End Method
  Function __randomSort%(arg1:Object, agr2:Object)
    Return Rand(-1, 1)
  End Function
End Type
Type TWave
  Field money%, steps:TList = New TList, currentStep:TStep, currentStepIndex% = 0
  Function Create:TWave(money%)
    Local result:TWave = New TWave
    result.money = money
    Return result
  End Function
  Method addStep(s:TStep)
    Self.steps.addLast s
  End Method
  Method start()
    If Self.steps.isEmpty() Then RuntimeError "no steps"
    Self.currentStepIndex = 0
    Self.currentStep = TStep(Self.steps.First())
    Self.currentStep.start
  End Method
  Method releaseEnemy%()
    If Self.currentStep.releaseEnemy() Then
      Self.currentStepIndex :+ 1
      If Self.currentStepIndex > Self.steps.Count()-1 Then
        Return 1
      EndIf
      Self.currentStep = TStep(Self.steps.valueAtIndex(Self.currentStepIndex))
      Self.currentStep.start
    EndIf
    Return 0
  End Method
  Method stop()
    .money :+ Self.money/(destruction+1)
    destruction = 0
  End Method
End Type
Type TLevel
  Global list:TList = New TList
  Method addToStory()
    Self.id = Self.list.Count()
    Self.list.addLast Self
  End Method
  Field waves:TList = New TList, map:TBank = CreateBank(), currentWaveIndex% = -1, currentWave:TWave
  Field id% = -1
  Method addWave(wave:TWave)
    Self.waves.addLast wave
  End Method
  Method readMap()
    ResizeBank Self.map, 0
    Local stream:TStream = CreateBankStream(Self.map)
    saveMap stream
    CloseStream stream
  End Method
  Method activateMap()
    Local stream:TStream = CreateBankStream(Self.map)
    loadMap stream
    CloseStream stream
  End Method
  Method startWave%()
    Self.currentWaveIndex :+ 1
    If Self.currentWaveIndex > Self.waves.Count()-1 Then Return 0
    Self.currentWave = TWave(Self.waves.valueAtIndex(Self.currentWaveIndex))
    Self.currentWave.start
    .currentWave = Self.currentWave
    Return 1
  End Method
  Method hasNextWave%()
    Return Self.currentWaveIndex < Self.waves.Count()-1
  End Method
Rem
Security-Hashes: The level-no will be saved with a hash
4 Bytes: level-no
4 Bytes: level-no ^ 2
4 Bytes: ~ level-no
4 Bytes: 2*level-no
4 Bytes: 2 ^ level-no
4 Bytes: sin(level-no)*1000
4 Bytes: 2 ^ level-no+3/(sin(level-no)+2)+314159
4 Bytes: level-no * 1234 ^ sin(level-no) + level-no - (level-no & $12345678)
End Rem
  Function hash:TBank(no%)
    Local result:TBank = CreateBank(32)
    PokeInt result, 0 , no
    PokeInt result, 4 , no^2
    PokeInt result, 8 , ~no
    PokeInt result, 12, 2*no
    PokeInt result, 16, 2^no
    PokeInt result, 20, Sin(no)*1000
    PokeInt result, 24, 2^no+3*(Sin(no)+2)+314159
    PokeInt result, 28, no*1234^Sin(no) + no - (no & $12345678)
    Return result
  End Function
  Function loadAll()
    Local index% = 1
    While FileType(STORY_DIR+"level"+index+".dat") = 1
      Local stream:TStream = ReadFile(STORY_DIR+"level"+index+".dat")
      Local level:TLevel = loadLevel(stream)
      level.addToStory
      CloseFile stream
      index :+ 1
    Wend
  End Function
End Type