Type TPlayer
  Const MAX_NO% = 4
  Const DATABASE$ = PLAYER_DIR+".data.dat"
  Global list:TPlayer[MAX_NO]
  Field name$, id%, currentLevel%, storyDone%
  Field starts%, uid%
  Rem
     To protect save-file: starts will be incremented every Object-creation, 
     uid is a random, static number
  End Rem
  Method New()
    Self.starts :+ 1
  End Method

  Method getHash:TBank()
    Local hash:Byte[32]
    '1. generate name-hash and fill bank with it
    Local name$ = Self.name+Chr(1)
    While Len(name) Mod 8
      name = name+Chr(0)
    Wend
    Local bytes:Byte[8]
    For Local i%=0 Until Len(name)/8
      Local part$ = name[i*8..(i+1)*8]
      For Local j%=0 To 7
        bytes[j] :+ Asc(part[j])*i
      Next
    Next
    For Local i%=0 To 3
      For Local j%=0 To 7
        hash[i*8+j] = bytes[j]
      Next
    Next
    '2. generate uid-hash and add it to the bank
    bytes[0] = Self.uid*42
    bytes[1] = Self.uid^2
    bytes[2] = Self.uid ~ $424242
    bytes[3] = Self.uid & $F0F0F0
    bytes[4] = 123+Self.uid
    bytes[5] = 1.5^Self.uid
    bytes[6] = 100/(128-Self.uid)
    bytes[7] = 1/(1.0/Self.uid+.001)
    For Local i%=0 To 3
      For Local j%=0 To 7
        hash[i*8+j] :+ bytes[j]*i
      Next
    Next
    '3. generate starts-hash and add it to the bank
    bytes[0] = ~Self.starts
    bytes[1] = Self.starts | $12481248
    bytes[2] = Self.starts+134
    bytes[3] = 180-Self.starts
    bytes[4] = Pi^Self.starts
    bytes[5] = $CAFEBABE ~ Self.starts
    bytes[6] = Self.starts*56
    bytes[7] = 1/(1.0/Self.starts+.0001)
    For Local i%=0 To 3
      For Local j%=0 To 7
        hash[i*8+j] :+ i^bytes[j]
      Next
    Next
    '4. generate id-hash and add it to the bank
    For Local i%=0 To 7
      bytes[i] = i*Self.id^i
    Next
    For Local i%=0 To 3
      For Local j%=0 To 7
        hash[i*8+j] :+ bytes[j] ~ i
      Next
    Next
    Local result:TBank = CreateBank(32)
    For Local i%=0 To 31
      PokeByte result, i, hash[i]
    Next
    Return result
  End Method
  Function loadAll()
    If FileType(Self.DATABASE) <> 1 Then Return
    Local stream:TStream = ReadFile(Self.DATABASE)
    While Not Eof(stream)
      loadPlayer stream
    Wend
    CloseFile stream
  End Function
  Function saveAll()
    Local stream:TStream = WriteFile(Self.DATABASE)
    For Local i%=0 To Self.MAX_NO-1
      If Self.list[i] Then savePlayer stream, Self.list[i]
    Next
    CloseFile stream
  End Function
  Function creationEnabled%()
    For Local i%=0 To Self.MAX_NO-1
      If Not Self.list[i] Then Return True
    Next
    Return False
  End Function
  Function Create:TPlayer(name$, id%)
    Local result:TPlayer = New TPlayer
    result.id = id
    result.uid = Rand($7FFFFFFF, $FFFFFFFF)
    result.name = name
    result.currentLevel = 1
    result.storyDone = 0
    result.starts = 0
    Self.list[id] = result
    Return result
  End Function

  Method winLevel(level%)
    If Self.currentLevel >= level+2 Then Return
    Self.currentLevel = level+2
    If level+2 >= TLevel.list.Count() Then Self.storyDone = True
  End Method
End Type