Rem
Save-Convention for maps:
  [LF-terminated String] Filename of background-image
  [Integrer] Number of waypoints
  [Integrer] Number of build-locked fields
  For each waypoint
    [Short] X-Coordinate
    [Short] Y-Coordinate
  Next
  For each build-lock
    [Short] X-Coordinate
    [Short] Y-Coordinate
  Next
End Rem
Function loadMap(stream:TStream)
  wayPoints.Clear()
  For Local x%=0 To TILESET_WIDTH-1
    For Local y%=0 To TILESET_HEIGHT-1
      buildLock[x][y] = False
    Next
  Next
  Local bgFile$ = ReadLine(stream)
  If bgFile.StartsWith(AppDir) Then
    bgFile = RealPath(bgFile)[Len(AppDir)+1..]
  EndIf
  Local noOfWayPoints% = ReadInt(stream)
  Local noOfLocks% = ReadInt(stream)
  Local i%
  For i=0 To noOfWayPoints-1
    Local x% = ReadShort(stream)
    Local y% = ReadShort(stream)
    TWayPoint.Create x, y
  Next
  For i=0 To noOfLocks-1
    buildLock[ReadShort(stream)][ReadShort(stream)] = True
  Next
  If MapContains(bgImages, bgFile) Then
    bg = TImage(MapValueForKey(bgImages, bgFile))
  Else
    bg = LoadImage(bgFile)
    SetImageHandle bg, 0, 0
    MapInsert bgImages, bgFile, bg
  EndIf
  bgFileName = bgFile
End Function
Function saveMap(stream:TStream)
  Local buildLocks:TList = New TList
  For Local x%=0 To TILESET_WIDTH-1
    For Local y%=0 To TILESET_HEIGHT-1
      If buildLock[x][y] Then
        Local point%[] = [x, y]
        buildLocks.addLast point
      EndIf
    Next
  Next
  Local bgFile$ = bgFileName
  If bgFile.StartsWith(AppDir) Then
    RuntimeError "old file found"
    bgFile = bgFile[Len(AppDir)+1..]
  EndIf
  WriteLine stream, bgFile
  WriteInt stream, wayPoints.Count()
  WriteInt stream, buildLocks.Count()
  For Local point:TWayPoint = EachIn wayPoints
    WriteShort stream, point.x
    WriteShort stream, point.y
  Next
  For Local point%[] = EachIn buildLocks
    WriteShort stream, point[0]
    WriteShort stream, point[1]
  Next
End Function
Rem
Save-Convention for waves:
  [Integrer] No. of steps
  [Integrer] money
  For each step
    [Short] Pause before (in frames)
    [Integrer] No. of enemies
    [Short] Time between two enemies *10
    [Byte] 1, if enemies come randomized
    For each enemy
      [Integrer] Index in TEnemyKind.list
    Next
  Next
End Rem
Function saveWave(stream:TStream, wave:TWave)
  WriteInt stream, wave.steps.Count()
  WriteInt stream, wave.money
  For Local s:TStep = EachIn wave.steps
    WriteShort stream, s.pauseBefore
    WriteInt stream, s.enemies.Count()
    WriteShort stream, s.enemyTime*10.0
    WriteByte stream, s.randomize
    For Local enemy:TEnemyKind = EachIn s.enemies
      WriteInt stream, enemy.id
    Next
  Next
End Function
Function loadWave:TWave(stream:TStream)
  Local result:TWave = New TWave
  Local noOfSteps% = ReadInt(stream)
  result.money = ReadInt(stream)
  For Local i%=0 Until noOfSteps
    Local s:TStep = New TStep
    s.pauseBefore = ReadShort(stream)
    Local noOfEnemies% = ReadInt(stream)
    s.enemyTime = ReadShort(stream)/10.0
    Assert s.enemyTime > 0.0, "Invalid enemy-time given"
    s.randomize = ReadByte(stream)
    For Local j%=0 Until noOfEnemies
      s.enemies.addLast TEnemyKind.list.valueAtIndex(ReadInt(stream))
    Next
    result.addStep s
  Next
  Return result
End Function
Rem
Save-Convention for levels:
[Integrer] No. of waves
For each wave
  [wave] wave as specified above
next
[map] map as specified at top
End Rem
Function saveLevel(stream:TStream, level:TLevel)
  WriteInt stream, level.waves.Count()
  For Local wave:TWave = EachIn level.waves
    saveWave stream, wave
  Next
  WriteBank level.map, stream, 0, BankSize(level.map)
End Function
Function loadLevel:TLevel(stream:TStream)
  Local result:TLevel = New TLevel
  Local noOfWaves% = ReadInt(stream)
  For Local i%=0 Until noOfWaves
    result.addWave loadWave(stream)
  Next
  result.map = CreateBank(StreamSize(stream)-StreamPos(stream))
  ReadBank result.map, stream, 0, StreamSize(stream)-StreamPos(stream)
  Return result
End Function
Function loadPlayer:TPlayer(stream:TStream)
  Local result:TPlayer = New TPlayer
  result.name = ReadLine(stream)
  result.id = ReadInt(stream)
  result.currentLevel = ReadInt(stream)
  Local hash:TBank = CreateBank(32)
  ReadBank hash, stream, 0, 32
  Local realHash:TBank = TLevel.hash(result.currentLevel)
  For Local i%=0 To 31
    If PeekByte(hash, i) <> PeekByte(realHash, i) Then showCheat
  Next
  result.uid    = ReadInt(stream)
  result.starts = ReadInt(stream)
  ReadBank hash, stream, 0, 32
  realHash = result.getHash()
  For Local i%=0 To 31
    If PeekByte(hash, i) <> PeekByte(realHash, i) Then showCheat
  Next
  result.storyDone = (result.currentLevel >= TLevel.list.Count())
  TPlayer.list[result.id] = result
  Return result
End Function
Function savePlayer(stream:TStream, player:TPlayer)
  WriteLine stream, player.name
  WriteInt stream, player.id
  WriteInt stream, player.currentLevel
  WriteBank TLevel.hash(player.currentLevel), stream, 0, 32
  WriteInt stream, player.uid
  WriteInt stream, player.starts
  WriteBank player.getHash(), stream, 0, 32
End Function