Type TTowerKind
  Global list:TList = New TList
  Method New()
    Self.list.addLast Self
  End Method
  Field range%, reloadTime%, power%, maxCapacity%, shootCapacity%, bulletSpeed#
  Field draw(instance:TTower), drawBullet(bullet:TBullet)
  Field price%
  Field tooltip$
  Field image:TImage
  Field detonating% = False, detonationRadius%
End Type

Type TTower
  Field x%, y%, reloadTime%, capacity%, kind:TTowerKind
  Field currentEnemy:TEnemy
  Field speedBoost%, rangeBoost%, reloadBoost%, capacityBoost%, efficiencyBoost%, powerBoost%
  Const BULLET_MAX_SPEED% = 30
  Const DETONATION_POWER_PART# = .5
  Const UPGRADE_SPEED%      = 0
  Const UPGRADE_RANGE%      = 1
  Const UPGRADE_RELOAD%     = 2
  Const UPGRADE_CAPACITY%   = 3
  Const UPGRADE_EFFICIENCY% = 4
  Const UPGRADE_POWER%      = 5
  Global names$[] = ["bullet speed", "range", "reload time", "max. capacity", "efficiency", "power"]


  Global list:TList = New TList
  Global index:TList[][]
  Function init()
    Self.index = Self.index[..TILESET_WIDTH]
    For Local i%=0 To TILESET_WIDTH-1
      Self.index[i] = Self.index[i][..TILESET_HEIGHT]
    Next
  End Function



  Function Create:TTower(x%, y%, kind:TTowerKind, sure% = False)
    If Not sure Then
      If buildLock[x][y] Then Return Null
      If Self.index[x][y] Then
        If Not Self.index[x][y].isEmpty() Then Return Null
      EndIf
      If TWire.index[x][y] Then
        If Not TWire.index[x][y].isEmpty() Then Return Null
      EndIf
    EndIf

    Local result:TTower = New TTower
    result.x = x
    result.y = y
    result.kind = kind
    Return result
  End Function

  Method draw()
    Self.kind.draw Self
    If Self.capacity = 0 Then
      SetColor 255, 255, 255
      DrawImage noElectricityImg, Self.x*TILE_SIZE, Self.y*TILE_SIZE
    EndIf
  End Method

  Method getFreeCapacity%()
    Return Self.getMaxCapacity()-Self.capacity
  End Method

  Method addCapacity(capacity%)
    Self.capacity :+ capacity
  End Method

  Method update()
    If Self.reloadTime <= 0 Then
      If Self.capacity >= Self.getShootCapacity() Then
        If Self.checkEnemy() Then
          Self.capacity :- Self.getShootCapacity()
          Self.shoot
          Self.reloadTime = Self.getReloadTime()
        EndIf
      EndIf
    Else
      Self.reloadTime :- 1
    EndIf
  End Method


  Function getByPosition:TList(x%, y%)
    Return Self.index[x][y]
  End Function

  Function getConnections:TList(x%, y%)
    Local result:TList = New TList
    Local move%[][] = [[0, 0], [0, 1], [1, 0], [0, -1], [-1, 0]]
    For Local i%=0 To 4
      If (x=0 And i=4) Or (x = TILESET_WIDTH -1 And i=2) Or ..
         (y=0 And i=3) Or (y = TILESET_HEIGHT-1 And i=1) Then Continue
      Local merge:TList = Self.index[x+move[i][0]][y+move[i][1]]
      If Not merge Then Continue
      For Local item:TTower = EachIn merge
        result.addLast item
      Next
    Next
    Return result
  End Function


  Method register()
    Local list:TList = TWire.getConnections(Self.x, Self.y)
    For Local item:TWire = EachIn list
      item.addConnection Self
    Next

    If Self.index[Self.x][Self.y]= Null Then Self.index[Self.x][Self.y]= New TList
    Self.index[Self.x][Self.y].addLast Self


    Self.list.addLast Self
  End Method

  Method checkEnemy%()
    Local useX% = Self.x*TILE_SIZE+TILE_SIZE/2
    Local useY% = Self.y*TILE_SIZE+TILE_SIZE/2
    If Self.currentEnemy Then
      If Not Self.currentEnemy.dead Then
        If (Self.currentEnemy.x-useX)^2+(Self.currentEnemy.y-useY)^2 < Self.getRange()^2 Then
          Return True
        EndIf
      EndIf
      Self.currentEnemy = Null
    EndIf
    For Local enemy:TEnemy = EachIn TEnemy.list
      If (enemy.x-useX)^2+(enemy.y-useY)^2 < Self.getRange()^2 Then
        Self.currentEnemy = enemy
        Return True
      EndIf
    Next
    Return False
  End Method

  Method shoot()
    TBullet.Create Self, Self.x*TILE_SIZE+TILE_SIZE/2, Self.y*TILE_SIZE+TILE_SIZE/2,..
                                   Self.currentEnemy.x, Self.currentEnemy.y, Self.currentEnemy
  End Method

  Method getBulletSpeed%()
    Return Self.kind.bulletSpeed+(Self.BULLET_MAX_SPEED-Self.kind.bulletSpeed)*(Self.speedBoost/5)
  End Method
  Method getRange%()
    Return Self.kind.range*(1.0+Self.rangeBoost/5.0)
  End Method
  Method getReloadTime%()
    Return Self.kind.reloadTime*(1.0-Self.reloadBoost/10.0)
  End Method
  Method getMaxCapacity%()
    Return Self.kind.maxCapacity*(1.0+Self.rangeBoost/5.0)
  End Method
  Method getShootCapacity%()
    Return Self.kind.shootCapacity*(1.0-Self.reloadBoost/10.0)
  End Method
  Method getPower%()
    Return Self.kind.power*(1.0+Self.rangeBoost/5.0)
  End Method
  Method getRelCapacity#()
    Return Float(Self.capacity)/Self.getMaxCapacity()
  End Method

  Method upgrade(property%)
    Select property
     Case UPGRADE_SPEED
      Self.speedBoost :+ 1
     Case UPGRADE_RANGE
      Self.rangeBoost :+ 1
     Case UPGRADE_RELOAD
      Self.reloadBoost :+ 1
     Case UPGRADE_CAPACITY
      Self.capacityBoost :+ 1
     Case UPGRADE_EFFICIENCY
      Self.efficiencyBoost :+ 1
     Case UPGRADE_POWER
      Self.powerBoost :+ 1
    End Select
  End Method
  Method getState%(property%)
    Select property
     Case UPGRADE_SPEED
      Return Self.speedBoost 
     Case UPGRADE_RANGE
      Return Self.rangeBoost 
     Case UPGRADE_RELOAD
      Return Self.reloadBoost 
     Case UPGRADE_CAPACITY
      Return Self.capacityBoost 
     Case UPGRADE_EFFICIENCY
      Return Self.efficiencyBoost 
     Case UPGRADE_POWER
      Return Self.powerBoost 
    End Select
  End Method
  Function getName$(property%)
    Return Self.names[property]
  End Function
  Function getPrice%(upgrade%)
    Return (Int(upgrade^1.4*100)/50)*50
  End Function
  Method getSellValue%()
    Local value% = Self.kind.price*.9
    For Local i%=0 To 5
      value :+ Self.getPrice(Self.getState(i))
    Next
    Return value
  End Method

  Function drawAll()
    For Local tower:TTower = EachIn Self.list
      tower.draw
    Next
  End Function


  Function updateAll()
    For Local tower:TTower = EachIn Self.list
      tower.update
    Next
  End Function

  Method checkDetonate(x%, y%)
    If Self.kind.detonating Then
      TDetonation.Create x, y, Self.kind.detonationRadius, Self.getPower()*Self.DETONATION_POWER_PART
    EndIf
  End Method
End Type

TTower.init