Type TWireKind
  Global list:TList = New TList
  Field leekage!
  Field maxCapacity%
  Field price%
  Field draw(instance:TWire)
  Field image:TImage

  Field production%     'Used for a power station
  Field tooltip$


  Method New()
    Self.list.addLast Self
  End Method
End Type


Type TWire
  Global list:TList = New TList
  Global index:TList[][]
  Field x%, y%
  Field kind:TWireKind
  Field connections:TList = New TList
  Field capacity#
  Field relCapacity#

  Function init()
    Self.index = Self.index[..TILESET_WIDTH]
    For Local i%=0 To TILESET_WIDTH-1
      Self.index[i] = Self.index[i][..TILESET_HEIGHT]
    Next
  End Function

  Function getByPosition:TList(x%, y%)
    Return Self.index[x][y]
  End Function

  Function getConnections:TList(x%, y%)
    Local result:TList = New TList
    Local move%[][] = [[0, 0], [0, 1], [1, 0], [0, -1], [-1, 0]]
    For Local i%=0 To 4
      If (x=0 And i=4) Or (x = TILESET_WIDTH -1 And i=2) Or ..
         (y=0 And i=3) Or (y = TILESET_HEIGHT-1 And i=1) Then Continue
      Local merge:TList = Self.index[x+move[i][0]][y+move[i][1]]
      If Not merge Then Continue
      For Local item:TWire = EachIn merge
        result.addLast item
      Next
    Next
    Return result
  End Function

  Function Create:TWire(x%, y%, kind:TWireKind)
    If buildLock[x][y] Then Return Null
    If Self.index[x][y] Then
      If Not Self.index[x][y].isEmpty() Then Return Null
    EndIf
    If TTower.index[x][y] Then
      If Not TTower.index[x][y].isEmpty() Then Return Null
    EndIf
    Local result:TWire = New TWire
    result.x = x
    result.y = y
    result.kind = kind
    list.addLast result
    Return result
  End Function

  Method register()
    Local list:TList = Self.getConnections(Self.x, Self.y)
    For Local item:TWire = EachIn list
      Self.addConnection item
    Next
    list = TTower.getConnections(Self.x, Self.y)
    For Local item:TTower = EachIn list
      Self.addConnection item
    Next


    If Self.index[Self.x][Self.y]= Null Then Self.index[Self.x][Self.y]= New TList
    Self.index[Self.x][Self.y].addLast Self

    Self.list.addLast Self
  End Method

  Method draw()
    Self.kind.draw Self
  End Method

  Method addConnection(conn:Object)
    Self.connections.addLast conn
    If TWire(conn) Then TWire(conn).connections.addLast Self
  End Method

  Method destroy()
    Local conn:Object
    For conn = EachIn Self.connections
      If TWire(conn) Then TWire(conn).connections.remove Self
    Next
    Self.list.remove Self
    Self.index[Self.x][Self.y].remove Self
  End Method

  Method update()
    Self.capacity :+ Self.kind.production    'For power-stations
    Local takers% = 0
    'Self.connections.Sort True, Self._ConnectSortFunc
    For Local conn:Object = EachIn Self.connections
      If TWire(conn) Then
        Local take% = TWire(conn).relCapacity < Self.relCapacity
        takers :+ take
      Else
        Local take% = TTower(conn).getRelCapacity() < Self.relCapacity
        takers :+ take
      EndIf
    Next
    If takers Then
      'takers :+ 1
      Local give% = Self.capacity/takers
      For Local conn:Object = EachIn Self.connections
        If TWire(conn) Then
          If TWire(conn).relCapacity > Self.relCapacity Then Continue 
          Local take% = Min(TWire(conn).kind.maxCapacity-TWire(conn).capacity, give)
          TWire(conn).capacity :+ take
          Self.capacity :- take
        Else
          If TTower(conn).getRelCapacity() > Self.relCapacity Then Continue 
          Local take% = Min(TTower(conn).getFreeCapacity(), give)
          TTower(conn).addCapacity take
          Self.capacity :- take
        EndIf
      Next
    EndIf
    Self.capacity = Min(Self.kind.maxCapacity, Self.capacity)*(1-Self.kind.leekage)
    Self.relCapacity = Float(Self.capacity)/Self.kind.maxCapacity
  End Method

  Function drawAll()
    SetAlpha WIRE_ALPHA
    For Local wire:TWire = EachIn Self.list
      wire.draw
    Next
    SetAlpha 1
  End Function

  Function updateAll()
    Self.list.Sort True, Self._generalSortfunc
    For Local wire:TWire = EachIn Self.list
      If KeyDown(KEY_S) Then
        Cls
        drawEverything
      EndIf
      wire.update
      If KeyDown(KEY_S) Then
        Flip
        WaitTimer timer
      EndIf
    Next
  End Function

  Function _ConnectSortfunc%(w1:Object, w2:Object)
    Local c1#, c2#
    If TWire(w1) Then
      c1 = TWire(w1).relCapacity
    Else
      c1 = TTower(w1).getRelCapacity()
    EndIf

    If TWire(w2) Then
      c2 = TWire(w2).relCapacity
    Else
      c2 = TTower(w2).getRelCapacity()
    EndIf
    If c1 = c2 Then Return 0
    If c1 < c2 Then Return -1
    Return 1
  End Function
  Function _generalSortfunc%(w1:Object, w2:Object)
    Local c1#, c2#
    If TWire(w1).relCapacity = TWire(w1).relCapacity Then Return 0
    If TWire(w1).relCapacity < TWire(w1).relCapacity Then Return -1
    Return 1
  End Function

End Type

TWire.init


